/** An enumeration of connection states */
const ConnectionStatus = Object.freeze({
	/** We are in the process of connecting */
	Connecting: 1,
	/** We have successfully connected */
	Connected: 2,
	/** The device went missing during the connection process */
	DeviceMissing: 3,
	/** The opener window went missing during the connection process */
	OpenerMissing: 4,
});

/** @type {MessagePort} */
let openerPort = null;
/** @type {WebSocket} */
let deviceConnection = null;
/** @type {String} */
let origin = null;

// set state to connected when the opener tells us so.
window.onmessage = function (event) {
	if (window.parent && event.source == window.parent && event.data === 'connected') {
		updateUI(ConnectionStatus.Connected);
	}
}

// cleanup the connections when the window is being closed and notify the opener.
window.onbeforeunload = function (event) {
	if (deviceConnection) {
		// remove onclose event so it doesn't mark this as an error
		deviceConnection.onclose = function () { };
		deviceConnection.close();
	}
	if (openerPort) {
		openerPort.close();
	}
	if (window.parent && window.parent) {
		window.parent.postMessage('closing', window.parent.location);
	}
};

updateUI(ConnectionStatus.Connecting);
const proxyUrl = new URL(window.parent.location);
const deviceServer = 'ws://' + window.location.hostname + ':1313';

deviceConnection = new WebSocket(deviceServer);
deviceConnection.onopen = function () {
	let openerChannel = new MessageChannel();
	openerPort = openerChannel.port1;
	// proxy messages received over the port
	openerPort.onmessage = function (evt) { deviceConnection.send(evt.data); };
	// initiate communication with the window that opened this one
	if (window.parent) {
		window.parent.postMessage('createChannel', window.parent.location, [openerChannel.port2]);
	} else {
		updateUI(ConnectionStatus.OpenerMissing);
	}
};
// proxy messages received over the device channel
deviceConnection.onmessage = function (evt) { openerPort.postMessage(evt.data); }
// if the connection closes for any reason, throw an error.
deviceConnection.onclose = function (close) { updateUI(ConnectionStatus.DeviceMissing); }

/**
 * Update the user interface for the specified status
 * @param {Number} status The status to reflect in the interface
 */
function updateUI(status) {
	// hide all of the icons first.
	document.getElementById('waitIcon').style.display = 'none';
	document.getElementById('connectedIcon').style.display = 'none';
	document.getElementById('errorIcon').style.display = 'none';

	switch (status) {
		case ConnectionStatus.Connecting:
			document.getElementById('statusHeader').innerText = 'Connecting Device';
			document.getElementById('statusMessage').innerHTML = '<a href="" onclick="self.close()">Close</a> this page to cancel.';
			document.getElementById('waitIcon').style.display = 'flex';
			break;
		case ConnectionStatus.Connected:
			document.getElementById('statusHeader').innerText = 'Connected';
			document.getElementById('statusMessage').innerHTML = 'You may <a href="" onclick="self.close()">close</a> this page.';
			document.getElementById('connectedIcon').style.display = 'flex';
			break;
		case ConnectionStatus.DeviceMissing:
			document.getElementById('statusHeader').innerText = 'Could not connect to device';
			document.getElementById('statusMessage').innerHTML = 'You may <a href="" onclick="self.close()">close</a> this page.';
			document.getElementById('errorIcon').style.display = 'flex';
			break;
		case ConnectionStatus.OpenerMissing:
			document.getElementById('statusHeader').innerText = 'Could not connect to host';
			document.getElementById('statusMessage').innerHTML = 'You may <a href="" onclick="self.close()">close</a> this page.';
			document.getElementById('errorIcon').style.display = 'flex';
			break;
	}
};